
import os
import sys
from itertools import chain
from string import Template as StringTemplate
import logging

from .compat import ExitStack, basestring

logger = logging.getLogger(__file__)

def is_variable(value):
    try:
        return bool(StringTemplate.pattern.search(value))
    except TypeError:
        return False

def iter_properties(iterable):
    """Split lines on '=' and ':=' ignoring blank lines and comments.
    """
    for line in iterable:
        if not line.isspace() and not line.startswith('#'):
            key, sep, val = line.partition(':=')
            if not sep:
                key, sep, val = line.partition('=')
            if not key.isspace() and sep:
                yield key.strip(), val.strip()

def interpolated(d, context=None):
    """Recursively interpolate dictionary values.

    The expected interpolation format is that defined within the stdlib
    `string.Template` class::

        $foo or ${foo}

    Will raise KeyError for missing keys and ValueError for invalid keys.
    """
    if context is None:
        context = dict(d)
    unresolved = {}
    for k,v in d.items():
        newval = StringTemplate(v).substitute(context)
        if StringTemplate.pattern.search(newval):
            unresolved[k] = newval
        else:
            context[k] = newval
    if unresolved:
        context.update(interpolated(unresolved, context))
    return context

def resolve(iterables, defaults=None, overrides=None, iterator=None):
    """
    Create a dictionary from an 'iterable of iterables' and interpolate the
    values within the result. Each iterable instance is expected to yield a
    key/value pair (ie. a 2-tuple of strings). Duplicate keys are allowed,
    but later keys will overrid earlier.
    """
    if not defaults:
        initial = {}
    else:
        # deepcopy?
        initial = dict(defaults)
    if overrides is None:
        overrides = {}
    iterator = iterator or iter_properties
    initial.update(dict(chain(*[iterator(item) for item in iterables])))
    result = interpolated(initial)
    result.update(overrides)
    return result

def resolve_files(files, defaults=None, overrides=None, iterator=None):
    """Create a a dictionary from one or more 'property' or 'env' files and
    interpolate the resulting values.
    """
    with ExitStack() as stack:
        iterables = []
        for f in files:
            if isinstance(f, basestring):
                iterables.append(stack.enter_context(open(f)))
            else:
                iterables.append(stack.enter_context(f))
        return resolve(iterables, defaults, overrides, iterator)




from __future__ import print_function
from __future__ import unicode_literals

import os
import sys
import json
import unittest

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from musette._environ import Environment, environ, resolve, resolve_files
from musette._environ import text_type

basename = os.path.basename
dirname = os.path.dirname
pathjoin = os.path.join
abspath = os.path.abspath

def filepath(relpath):
    return pathjoin(dirname(abspath(__file__)), "data", relpath.lstrip('/'))

class BaseTests(unittest.TestCase):

    URL = 'http://www.google.com/'
    POSTGRES = 'postgres://uf07k1i6d8ia0v:wegauwhgeuioweg@ec2-107-21-253-135.compute-1.amazonaws.com:5431/d8r82722r2kuvn'
    MYSQL = 'mysql://bea6eb025ca0d8:69772142@us-cdbr-east.cleardb.com/heroku_97681db3eff7580?reconnect=true'
    MYSQLGIS = 'mysqlgis://user:password@127.0.0.1/some_database'
    SQLITE = 'sqlite:////full/path/to/your/database/file.sqlite'
    MEMCACHE = 'memcache://127.0.0.1:11211'
    REDIS = 'rediscache://127.0.0.1:6379:1?client_class=redis_cache.client.DefaultClient&password=secret'
    EMAIL = 'smtps://user@domain.com:password@smtp.example.com:587'
    JSON = dict(one='bar', two=2, three=33.44)
    DICT = dict(foo='bar', test='on')
    PATH = '/home/dev'

    @classmethod
    def generateData(cls):
        return dict(STR_VAR='bar',
                    INT_VAR='42',
                    FLOAT_VAR='33.3',
                    FLOAT_COMMA_VAR='33,3',
                    FLOAT_STRANGE_VAR1='123,420,333.3',
                    FLOAT_STRANGE_VAR2='123.420.333,3',
                    BOOL_TRUE_VAR='1',
                    BOOL_TRUE_VAR2='True',
                    BOOL_FALSE_VAR='0',
                    BOOL_FALSE_VAR2='False',
                    PROXIED_VAR='$STR_VAR',
                    INT_LIST='42,33',
                    STR_LIST_WITH_SPACES=' foo,  bar',
                    EMPTY_LIST='',
                    DICT_VAR='foo=bar,test=on',
                    DATABASE_URL=cls.POSTGRES,
                    DATABASE_MYSQL_URL=cls.MYSQL,
                    DATABASE_MYSQL_GIS_URL=cls.MYSQLGIS,
                    DATABASE_SQLITE_URL=cls.SQLITE,
                    CACHE_URL=cls.MEMCACHE,
                    CACHE_REDIS=cls.REDIS,
                    EMAIL_URL=cls.EMAIL,
                    URL_VAR=cls.URL,
                    JSON_VAR=json.dumps(cls.JSON),
                    PATH_VAR=cls.PATH,
                    A_SECRET_VAR='my secret key',
                    A_PASSWORD_VAR='qwerty123',
                   )

    def setUp(self):
        self.env = Environment(self.generateData())

    def assertTypeAndValue(self, type_, expected, actual):
        self.assertEqual(type_, type(actual))
        self.assertEqual(expected, actual)


class EnvTests(BaseTests):

    def test_not_present_with_default(self):
        self.assertEqual(3, self.env('not_present', default=3))

    def test_not_present_without_default(self):
        self.assertRaises(KeyError, self.env, 'not_present')

    def test_str(self):
        self.assertTypeAndValue(text_type, 'bar', self.env('STR_VAR'))
        self.assertTypeAndValue(text_type, 'bar', self.env.str('STR_VAR'))

    def test_int(self):
        self.assertTypeAndValue(int, 42, self.env('INT_VAR', cast=int))
        self.assertTypeAndValue(int, 42, self.env.int('INT_VAR'))

    def test_int_with_none_default(self):
        self.assertTrue(self.env('NOT_PRESENT_VAR', cast=int, default=None) is None)

    def test_float(self):
        self.assertTypeAndValue(float, 33.3, self.env('FLOAT_VAR', cast=float))
        self.assertTypeAndValue(float, 33.3, self.env.float('FLOAT_VAR'))

        self.assertTypeAndValue(float, 33.3, self.env('FLOAT_COMMA_VAR', cast=float))
        self.assertTypeAndValue(float, 123420333.3, self.env('FLOAT_STRANGE_VAR1', cast=float))
        self.assertTypeAndValue(float, 123420333.3, self.env('FLOAT_STRANGE_VAR2', cast=float))

    def test_bool_true(self):
        self.assertTypeAndValue(bool, True, self.env('BOOL_TRUE_VAR', cast=bool))
        self.assertTypeAndValue(bool, True, self.env('BOOL_TRUE_VAR2', cast=bool))
        self.assertTypeAndValue(bool, True, self.env.bool('BOOL_TRUE_VAR'))

    def test_bool_false(self):
        self.assertTypeAndValue(bool, False, self.env('BOOL_FALSE_VAR', cast=bool))
        self.assertTypeAndValue(bool, False, self.env('BOOL_FALSE_VAR2', cast=bool))
        self.assertTypeAndValue(bool, False, self.env.bool('BOOL_FALSE_VAR'))

    def test_proxied_value(self):
        self.assertTypeAndValue(text_type, 'bar', self.env('PROXIED_VAR'))

    def test_int_list(self):
        self.assertTypeAndValue(list, [42, 33], self.env('INT_LIST', cast=[int]))
        self.assertTypeAndValue(list, [42, 33], self.env.list('INT_LIST', int))

    def test_str_list_with_spaces(self):
        self.assertTypeAndValue(list, [' foo', '  bar'],
                                self.env('STR_LIST_WITH_SPACES', cast=[text_type]))
        self.assertTypeAndValue(list, [' foo', '  bar'],
                                self.env.list('STR_LIST_WITH_SPACES'))

    def test_empty_list(self):
        self.assertTypeAndValue(list, [], self.env('EMPTY_LIST', cast=[int]))

    def test_dict_value(self):
        self.assertTypeAndValue(dict, self.DICT, self.env.dict('DICT_VAR'))

    def test_dict_parsing(self):

        self.assertEqual({'a': '1'}, self.env.parse_value('a=1', dict))
        self.assertEqual({'a': 1}, self.env.parse_value('a=1', dict(value=int)))
        self.assertEqual({'a': ['1', '2', '3']}, self.env.parse_value('a=1,2,3', dict(value=[text_type])))
        self.assertEqual({'a': [1, 2, 3]}, self.env.parse_value('a=1,2,3', dict(value=[int])))
        self.assertEqual({'a': 1, 'b': [1.1, 2.2], 'c': 3},
                         self.env.parse_value('a=1;b=1.1,2.2;c=3', dict(value=int, cast=dict(b=[float]))))

    def test_url_value(self):
        url = self.env.url('URL_VAR')
        self.assertEqual(url.__class__, self.env.URL_CLASS)
        self.assertEqual(url.geturl(), self.URL)

    def test_db_url_value(self):
        pg_config = self.env.db()
        self.assertEqual(pg_config['ENGINE'], 'django.db.backends.postgresql_psycopg2')
        self.assertEqual(pg_config['NAME'], 'd8r82722r2kuvn')
        self.assertEqual(pg_config['HOST'], 'ec2-107-21-253-135.compute-1.amazonaws.com')
        self.assertEqual(pg_config['USER'], 'uf07k1i6d8ia0v')
        self.assertEqual(pg_config['PASSWORD'], 'wegauwhgeuioweg')
        self.assertEqual(pg_config['PORT'], 5431)

        mysql_config = self.env.db('DATABASE_MYSQL_URL')
        self.assertEqual(mysql_config['ENGINE'], 'django.db.backends.mysql')
        self.assertEqual(mysql_config['NAME'], 'heroku_97681db3eff7580')
        self.assertEqual(mysql_config['HOST'], 'us-cdbr-east.cleardb.com')
        self.assertEqual(mysql_config['USER'], 'bea6eb025ca0d8')
        self.assertEqual(mysql_config['PASSWORD'], '69772142')
        self.assertEqual(mysql_config['PORT'], None)
        
        mysql_gis_config = self.env.db('DATABASE_MYSQL_GIS_URL')
        self.assertEqual(mysql_gis_config['ENGINE'], 'django.contrib.gis.db.backends.mysql')
        self.assertEqual(mysql_gis_config['NAME'], 'some_database')
        self.assertEqual(mysql_gis_config['HOST'], '127.0.0.1')
        self.assertEqual(mysql_gis_config['USER'], 'user')
        self.assertEqual(mysql_gis_config['PASSWORD'], 'password')
        self.assertEqual(mysql_gis_config['PORT'], None)

        sqlite_config = self.env.db('DATABASE_SQLITE_URL')
        self.assertEqual(sqlite_config['ENGINE'], 'django.db.backends.sqlite3')
        self.assertEqual(sqlite_config['NAME'], '/full/path/to/your/database/file.sqlite')

    def test_cache_url_value(self):

        cache_config = self.env.cache_url()
        self.assertEqual(cache_config['BACKEND'], 'django.core.cache.backends.memcached.MemcachedCache')
        self.assertEqual(cache_config['LOCATION'], '127.0.0.1:11211')

        redis_config = self.env.cache_url('CACHE_REDIS')
        self.assertEqual(redis_config['BACKEND'], 'redis_cache.cache.RedisCache')
        self.assertEqual(redis_config['LOCATION'], '127.0.0.1:6379:1')
        self.assertEqual(redis_config['OPTIONS'], {
            'CLIENT_CLASS': 'redis_cache.client.DefaultClient',
            'PASSWORD': 'secret',
        })

    def test_email_url_value(self):

        email_config = self.env.email_url()
        self.assertEqual(email_config['EMAIL_BACKEND'],
                'django.core.mail.backends.smtp.EmailBackend')
        self.assertEqual(email_config['EMAIL_HOST'], 'smtp.example.com')
        self.assertEqual(email_config['EMAIL_HOST_PASSWORD'], 'password')
        self.assertEqual(email_config['EMAIL_HOST_USER'], 'user@domain.com')
        self.assertEqual(email_config['EMAIL_PORT'], 587)
        self.assertEqual(email_config['EMAIL_USE_TLS'], True)


    def test_json_value(self):
        self.assertEqual(self.JSON, self.env.json('JSON_VAR'))

class FileEnvTests(EnvTests):

    def setUp(self):
        self.env = Environment(self.generateData())
        file_path = filepath('test_env.txt')
        self.env.read_env(file_path, PATH_VAR="/another/path")

    def test_path(self):
        self.assertNotEqual(self.PATH, self.env('PATH_VAR'))

class OsEnvironTests(unittest.TestCase):

    def test_singleton_environ(self):
        env = Environment()
        self.assertTrue(env._environ is os.environ)

    def test_environ_mutation(self):
        os.environ['MUSETTE_TEST_KEY'] = 'musette'
        env = Environment()
        self.assertEqual(env['MUSETTE_TEST_KEY'], 'musette')
        env['MUSETTE_TEST_KEY'] = 'mutated'
        self.assertEqual(env['MUSETTE_TEST_KEY'], 'mutated')
        self.assertEqual(os.environ['MUSETTE_TEST_KEY'], 'mutated')
        del env['MUSETTE_TEST_KEY']
        with self.assertRaises(KeyError):
            env['MUSETTE_TEST_KEY']
        with self.assertRaises(KeyError):
            os.environ['MUSETTE_TEST_KEY']

class AltEnvironTests(unittest.TestCase):

    def setUp(self):
        self._environ = {}
        self.env = Environment(self._environ)

    def test_singleton_environ(self):
        try:
            os_environ = os.environ.data
        except AttributeError:
            os_environ = os.environ._data
        self.assertFalse(self.env._environ is os_environ)

class DictionaryInterfaceTests(BaseTests):

    def setUp(self):
        self.env = Environment(
            self.generateData(),
            INT_VAR=int,
            NOT_PRESENT_VAR=(float, 33.3),
            STR_VAR=text_type,
            INT_LIST=[int],
            DEFAULT_LIST=([int], [2])
        )

    def test_get(self):
        self.assertTypeAndValue(int, 42, self.env['INT_VAR'])
        self.assertTypeAndValue(int, 42, self.env.get('INT_VAR'))

    def test_set_and_get(self):
        self.env['INT_VAR'] = 24
        self.assertTypeAndValue(int, 24, self.env['INT_VAR'])
        self.assertTypeAndValue(int, 24, self.env.get('INT_VAR'))
        self.env['INT_VAR'] = '99'
        self.assertTypeAndValue(int, 99, self.env['INT_VAR'])
        self.assertTypeAndValue(int, 99, self.env.get('INT_VAR'))

    def test_length(self):
        self.assertEqual(len(self.env), 27)

    def test_keys_method(self):
        self.assertEqual(set(self.env.keys()), set(self.generateData().keys()))

    def test_copy(self):
        copied = self.env.copy()
        self.assertTrue(type(self.env) is type(copied))
        self.assertEqual(len(self.env), len(copied))

class SchemaEnvTests(BaseTests):

    def setUp(self):
        self.env = Environment(
            self.generateData(),
            INT_VAR=int,
            NOT_PRESENT_VAR=(float, 33.3),
            STR_VAR=text_type,
            INT_LIST=[int],
            DEFAULT_LIST=([int], [2])
        )

    def test_schema(self):
        self.assertTypeAndValue(int, 42, self.env('INT_VAR'))
        self.assertTypeAndValue(float, 33.3, self.env('NOT_PRESENT_VAR'))

        self.assertTypeAndValue(text_type, 'bar', self.env('STR_VAR'))
        self.assertTypeAndValue(text_type, 'foo', self.env('NOT_PRESENT2', default='foo'))

        self.assertTypeAndValue(list, [42, 33], self.env('INT_LIST'))
        self.assertTypeAndValue(list, [2], self.env('DEFAULT_LIST'))

        # Override schema in this one case
        self.assertTypeAndValue(text_type, '42', self.env('INT_VAR', cast=text_type))


class DatabaseTestSuite(unittest.TestCase):

    def test_postgres_parsing(self):
        url = 'postgres://uf07k1i6d8ia0v:wegauwhgeuioweg@ec2-107-21-253-135.compute-1.amazonaws.com:5431/d8r82722r2kuvn'
        url = environ.db_url_config(url)

        self.assertEqual(url['ENGINE'], 'django.db.backends.postgresql_psycopg2')
        self.assertEqual(url['NAME'], 'd8r82722r2kuvn')
        self.assertEqual(url['HOST'], 'ec2-107-21-253-135.compute-1.amazonaws.com')
        self.assertEqual(url['USER'], 'uf07k1i6d8ia0v')
        self.assertEqual(url['PASSWORD'], 'wegauwhgeuioweg')
        self.assertEqual(url['PORT'], 5431)

    def test_postgis_parsing(self):
        url = 'postgis://uf07k1i6d8ia0v:wegauwhgeuioweg@ec2-107-21-253-135.compute-1.amazonaws.com:5431/d8r82722r2kuvn'
        url = environ.db_url_config(url)

        self.assertEqual(url['ENGINE'], 'django.contrib.gis.db.backends.postgis')
        self.assertEqual(url['NAME'], 'd8r82722r2kuvn')
        self.assertEqual(url['HOST'], 'ec2-107-21-253-135.compute-1.amazonaws.com')
        self.assertEqual(url['USER'], 'uf07k1i6d8ia0v')
        self.assertEqual(url['PASSWORD'], 'wegauwhgeuioweg')
        self.assertEqual(url['PORT'], 5431)

    def test_mysql_gis_parsing(self):
        url = 'mysqlgis://uf07k1i6d8ia0v:wegauwhgeuioweg@ec2-107-21-253-135.compute-1.amazonaws.com:5431/d8r82722r2kuvn'
        url = environ.db_url_config(url)

        self.assertEqual(url['ENGINE'], 'django.contrib.gis.db.backends.mysql')
        self.assertEqual(url['NAME'], 'd8r82722r2kuvn')
        self.assertEqual(url['HOST'], 'ec2-107-21-253-135.compute-1.amazonaws.com')
        self.assertEqual(url['USER'], 'uf07k1i6d8ia0v')
        self.assertEqual(url['PASSWORD'], 'wegauwhgeuioweg')
        self.assertEqual(url['PORT'], 5431)

    def test_cleardb_parsing(self):
        url = 'mysql://bea6eb025ca0d8:69772142@us-cdbr-east.cleardb.com/heroku_97681db3eff7580?reconnect=true'
        url = environ.db_url_config(url)

        self.assertEqual(url['ENGINE'], 'django.db.backends.mysql')
        self.assertEqual(url['NAME'], 'heroku_97681db3eff7580')
        self.assertEqual(url['HOST'], 'us-cdbr-east.cleardb.com')
        self.assertEqual(url['USER'], 'bea6eb025ca0d8')
        self.assertEqual(url['PASSWORD'], '69772142')
        self.assertEqual(url['PORT'], None)

    def test_empty_sqlite_url(self):
        url = 'sqlite://'
        url = environ.db_url_config(url)

        self.assertEqual(url['ENGINE'], 'django.db.backends.sqlite3')
        self.assertEqual(url['NAME'], ':memory:')

    def test_memory_sqlite_url(self):
        url = 'sqlite://:memory:'
        url = environ.db_url_config(url)

        self.assertEqual(url['ENGINE'], 'django.db.backends.sqlite3')
        self.assertEqual(url['NAME'], ':memory:')

    def test_database_options_parsing(self):
        url = 'postgres://user:pass@host:1234/dbname?conn_max_age=600'
        url = environ.db_url_config(url)
        self.assertEqual(url['CONN_MAX_AGE'], 600)

        url = 'mysql://user:pass@host:1234/dbname?init_command=SET storage_engine=INNODB'
        url = environ.db_url_config(url)
        self.assertEqual(url['OPTIONS'], {
            'init_command': 'SET storage_engine=INNODB',
        })

    def test_database_ldap_url(self):
        url = 'ldap://cn=admin,dc=nodomain,dc=org:some_secret_password@ldap.nodomain.org/'
        url = environ.db_url_config(url)

        self.assertEqual(url['ENGINE'], 'ldapdb.backends.ldap')
        self.assertEqual(url['HOST'], 'ldap.nodomain.org')
        self.assertEqual(url['PORT'], None)
        self.assertEqual(url['NAME'], 'ldap://ldap.nodomain.org')
        self.assertEqual(url['USER'], 'cn=admin,dc=nodomain,dc=org')
        self.assertEqual(url['PASSWORD'], 'some_secret_password')

class CacheTestSuite(unittest.TestCase):

    def test_memcache_parsing(self):
        url = 'memcache://127.0.0.1:11211'
        url = environ.cache_url_config(url)

        self.assertEqual(url['BACKEND'], 'django.core.cache.backends.memcached.MemcachedCache')
        self.assertEqual(url['LOCATION'], '127.0.0.1:11211')

    def test_memcache_pylib_parsing(self):
        url = 'pymemcache://127.0.0.1:11211'
        url = environ.cache_url_config(url)

        self.assertEqual(url['BACKEND'], 'django.core.cache.backends.memcached.PyLibMCCache')
        self.assertEqual(url['LOCATION'], '127.0.0.1:11211')

    def test_memcache_multiple_parsing(self):
        url = 'memcache://172.19.26.240:11211,172.19.26.242:11212'
        url = environ.cache_url_config(url)

        self.assertEqual(url['BACKEND'], 'django.core.cache.backends.memcached.MemcachedCache')
        self.assertEqual(url['LOCATION'], ['172.19.26.240:11211', '172.19.26.242:11212'])

    def test_memcache_socket_parsing(self):
        url = 'memcache:///tmp/memcached.sock'
        url = environ.cache_url_config(url)

        self.assertEqual(url['BACKEND'], 'django.core.cache.backends.memcached.MemcachedCache')
        self.assertEqual(url['LOCATION'], 'unix:/tmp/memcached.sock')

    def test_dbcache_parsing(self):
        url = 'dbcache://my_cache_table'
        url = environ.cache_url_config(url)

        self.assertEqual(url['BACKEND'], 'django.core.cache.backends.db.DatabaseCache')
        self.assertEqual(url['LOCATION'], 'my_cache_table')

    def test_filecache_parsing(self):
        url = 'filecache:///var/tmp/django_cache'
        url = environ.cache_url_config(url)

        self.assertEqual(url['BACKEND'], 'django.core.cache.backends.filebased.FileBasedCache')
        self.assertEqual(url['LOCATION'], '/var/tmp/django_cache')

    def test_filecache_windows_parsing(self):
        url = 'filecache://C:/foo/bar'
        url = environ.cache_url_config(url)

        self.assertEqual(url['BACKEND'], 'django.core.cache.backends.filebased.FileBasedCache')
        self.assertEqual(url['LOCATION'], 'C:/foo/bar')

    def test_locmem_parsing(self):
        url = 'locmemcache://'
        url = environ.cache_url_config(url)

        self.assertEqual(url['BACKEND'], 'django.core.cache.backends.locmem.LocMemCache')
        self.assertEqual(url['LOCATION'], '')

    def test_locmem_named_parsing(self):
        url = 'locmemcache://unique-snowflake'
        url = environ.cache_url_config(url)

        self.assertEqual(url['BACKEND'], 'django.core.cache.backends.locmem.LocMemCache')
        self.assertEqual(url['LOCATION'], 'unique-snowflake')

    def test_dummycache_parsing(self):
        url = 'dummycache://'
        url = environ.cache_url_config(url)

        self.assertEqual(url['BACKEND'], 'django.core.cache.backends.dummy.DummyCache')
        self.assertEqual(url['LOCATION'], '')

    def test_redis_parsing(self):
        url = 'rediscache://127.0.0.1:6379:1?client_class=redis_cache.client.DefaultClient&password=secret'
        url = environ.cache_url_config(url)

        self.assertEqual(url['BACKEND'], 'redis_cache.cache.RedisCache')
        self.assertEqual(url['LOCATION'], '127.0.0.1:6379:1')
        self.assertEqual(url['OPTIONS'], {
            'CLIENT_CLASS': 'redis_cache.client.DefaultClient',
            'PASSWORD': 'secret',
        })

    def test_redis_socket_parsing(self):
        url = 'rediscache:///path/to/socket:1'
        url = environ.cache_url_config(url)
        self.assertEqual(url['BACKEND'], 'redis_cache.cache.RedisCache')
        self.assertEqual(url['LOCATION'], 'unix:/path/to/socket:1')

    def test_options_parsing(self):
        url = 'filecache:///var/tmp/django_cache?timeout=60&max_entries=1000&cull_frequency=0'
        url = environ.cache_url_config(url)

        self.assertEqual(url['BACKEND'], 'django.core.cache.backends.filebased.FileBasedCache')
        self.assertEqual(url['LOCATION'], '/var/tmp/django_cache')
        self.assertEqual(url['TIMEOUT'], 60)
        self.assertEqual(url['OPTIONS'], {
            'MAX_ENTRIES': 1000,
            'CULL_FREQUENCY': 0,
        })

    def test_custom_backend(self):
        url = 'memcache://127.0.0.1:5400?foo=option&bars=9001'
        backend = 'redis_cache.cache.RedisCache'
        url = environ.cache_url_config(url, backend)

        self.assertEqual(url['BACKEND'], backend)
        self.assertEqual(url['LOCATION'], '127.0.0.1:5400')
        self.assertEqual(url['OPTIONS'], {
            'FOO': 'option',
            'BARS': 9001,
        })


class EmailTests(unittest.TestCase):

    def test_smtp_parsing(self):
        url = 'smtps://user@domain.com:password@smtp.example.com:587'
        url = environ.email_url_config(url)

        self.assertEqual(url['EMAIL_BACKEND'],
                'django.core.mail.backends.smtp.EmailBackend')
        self.assertEqual(url['EMAIL_HOST'], 'smtp.example.com')
        self.assertEqual(url['EMAIL_HOST_PASSWORD'], 'password')
        self.assertEqual(url['EMAIL_HOST_USER'], 'user@domain.com')
        self.assertEqual(url['EMAIL_PORT'], 587)
        self.assertEqual(url['EMAIL_USE_TLS'], True)


class InterpolationTests(unittest.TestCase):

    def test_empty_input(self):
        lines = '''
        '''.splitlines()
        d = resolve([lines])
        self.assertTrue(type(d) is dict)
        self.assertTrue(len(d) == 0)

    def test_default_separators(self):
        lines = '''
        foo := bar
        fee:= fum
        fab :=day
        null :=
        '''.splitlines()
        d = resolve([lines])
        self.assertEqual(d['foo'], 'bar')
        self.assertEqual(d['fee'], 'fum')
        self.assertEqual(d['fab'], 'day')
        self.assertEqual(d['null'], '')
        lines = '''
        foo = bar
        fee= fum
        fab =day
        null =
        '''.splitlines()
        d = resolve([lines])
        self.assertEqual(d['foo'], 'bar')
        self.assertEqual(d['fee'], 'fum')
        self.assertEqual(d['fab'], 'day')
        self.assertEqual(d['null'], '')

    def test_multiple_input(self):
        common = '''
        foo := bar
        fee:= fum
        fab :=day
        null :=
        '''.splitlines()
        env_overrides = '''
        password := mysecret
        fab := night
        null := nolongernull
        '''.splitlines()
        d = resolve([common, env_overrides])
        self.assertEqual(d['foo'], 'bar')
        self.assertEqual(d['fee'], 'fum')
        self.assertEqual(d['fab'], 'night')
        self.assertEqual(d['null'], 'nolongernull')
        self.assertEqual(d['password'], 'mysecret')

    def test_interpolation(self):
        lines = '''
        foo := bar
        fat := cat
        fee := DB_${foo}
        fab := ${fee} TABLE_${foo}
        baf := TABLE_${foo} $fee
        fit := $fat ${fab}
        tif := $fab $fat
        '''.splitlines()
        d = resolve([lines])
        self.assertEqual(d['foo'], 'bar')
        self.assertEqual(d['fee'], 'DB_bar')
        self.assertEqual(d['fab'], 'DB_bar TABLE_bar')
        self.assertEqual(d['baf'], 'TABLE_bar DB_bar')
        self.assertEqual(d['fit'], 'cat DB_bar TABLE_bar')
        self.assertEqual(d['tif'], 'DB_bar TABLE_bar cat')

    def test_bad_input(self):
        lines = '''
        foo := bar
        fee := DB_$typo
        '''.splitlines()
        self.assertRaises(KeyError, resolve, [lines])

        lines = '''
        2foo := bar
        fee := DB_$2foo
        '''.splitlines()
        self.assertRaises(ValueError, resolve, [lines])

    def test_defaults(self):
        lines = '''
        foo := bar
        fee := DB_$typo
        '''.splitlines()
        d = resolve([lines], {'typo': 'TEST'})
        self.assertEqual(d['foo'], 'bar')
        self.assertEqual(d['fee'], 'DB_TEST')

    def test_overrides(self):
        lines = '''
        foo := bar
        fee := DB_$typo
        '''.splitlines()
        d = resolve([lines], {'typo': 'TEST'}, {'fee': 'OVERRIDE'})
        self.assertEqual(d['foo'], 'bar')
        self.assertEqual(d['fee'], 'OVERRIDE')

    def test_file_input(self):
        infiles = [filepath("common.properties"), filepath("env.properties")]
        d = resolve_files(infiles)
        self.assertEqual(d['foo'], 'TEST')
        self.assertEqual(d['fee'], 'DB_TEST')
        self.assertEqual(d['fab'], 'DB_TEST TABLE_TEST')
        self.assertEqual(d['baf'], 'TABLE_TEST DB_TEST')
        self.assertEqual(d['fit'], 'cat DB_TEST TABLE_TEST')
        self.assertEqual(d['tif'], 'DB_TEST TABLE_TEST cat')

    def test_read_method(self):
        ENVIRON = {}
        env = Environment(ENVIRON)
        infiles = [filepath("common.properties"), filepath("env.properties")]
        env.read(infiles)
        self.assertEqual(ENVIRON['foo'], 'TEST')
        self.assertEqual(ENVIRON['fee'], 'DB_TEST')
        self.assertEqual(ENVIRON['fab'], 'DB_TEST TABLE_TEST')
        self.assertEqual(ENVIRON['baf'], 'TABLE_TEST DB_TEST')
        self.assertEqual(ENVIRON['fit'], 'cat DB_TEST TABLE_TEST')
        self.assertEqual(ENVIRON['tif'], 'DB_TEST TABLE_TEST cat')

class MoreInterpolationTests(unittest.TestCase):

    def test_set_and_get_variable_values(self):
        ENVIRON = {}
        env = Environment(ENVIRON)
        env['PATH'] = '$CURRENT/path/to/file'
        self.assertEqual(env['PATH'], '$CURRENT/path/to/file')
        env['CURRENT'] = '/home/user'
        self.assertEqual(env['PATH'], '/home/user/path/to/file')

    def test_set_and_get_nested_variable_values(self):
        ENVIRON = {}
        env = Environment(ENVIRON)
        env['PATH'] = '$CURRENT/path/to/file/$NAME'
        self.assertEqual(env['PATH'], '$CURRENT/path/to/file/$NAME')
        env['CURRENT'] = '${ROOT}/instance'
        # unresolved
        self.assertEqual(env['CURRENT'], '${ROOT}/instance')
        self.assertEqual(env['PATH'], '$CURRENT/path/to/file/$NAME')
        env['ROOT'] = '/opt'
        # still unresolved
        self.assertEqual(env['CURRENT'], '${ROOT}/instance')
        self.assertEqual(env['PATH'], '$CURRENT/path/to/file/$NAME')
        env['NAME'] = 'setup.py'
        # resolved
        self.assertEqual(env['CURRENT'], '/opt/instance')
        self.assertEqual(env['PATH'], '/opt/instance/path/to/file/setup.py')

    def test_set_and_get_nested_variable_values_again(self):
        ENVIRON = {}
        env = Environment(ENVIRON)
        env['PATH'] = '$CURRENT/path/to/file/$NAME'
        self.assertEqual(env['PATH'], '$CURRENT/path/to/file/$NAME')
        env['CURRENT'] = '${ROOT}/instance'
        # unresolved
        self.assertEqual(env['CURRENT'], '${ROOT}/instance')
        self.assertEqual(env['PATH'], '$CURRENT/path/to/file/$NAME')
        env['NAME'] = 'setup.py'
        # still unresolved
        self.assertEqual(env['CURRENT'], '${ROOT}/instance')
        self.assertEqual(env['PATH'], '$CURRENT/path/to/file/$NAME')
        env['ROOT'] = '/opt'
        # resolved
        self.assertEqual(env['CURRENT'], '/opt/instance')
        self.assertEqual(env['PATH'], '/opt/instance/path/to/file/setup.py')

    def test_variables_in_initial_dict(self):
        env = Environment({'INSTALL_ROOT': '$PWD', 'PATH': '$INSTALL_ROOT/path'})
        self.assertEqual(env['INSTALL_ROOT'], '$PWD')
        self.assertEqual(env['PATH'], '$INSTALL_ROOT/path')
        env['PWD'] = '/home/user'
        self.assertEqual(env['INSTALL_ROOT'], '/home/user')
        self.assertEqual(env['PATH'], '/home/user/path')

class PrettyPrintTests(BaseTests):

    def test_pprint(self):
        from musette.compat import BytesIO
        stream = BytesIO()
        self.env.pprint(stream=stream, maxlines=1)
        expected = b'''
A_PASSWORD_VAR = ********

'''
        self.assertEqual(stream.getvalue(), expected)

        stream.seek(0)
        self.env.pprint(stream=stream, maxlines=2)
        expected = b'''
A_PASSWORD_VAR = ********
A_SECRET_VAR = ********

'''
        self.assertEqual(stream.getvalue(), expected)

        stream.seek(0)
        self.env.pprint(stream=stream, maxlines=13)
        expected = b'''
A_PASSWORD_VAR = ********
A_SECRET_VAR = ********

BOOL_FALSE_VAR = 0
BOOL_FALSE_VAR2 = False
BOOL_TRUE_VAR = 1
BOOL_TRUE_VAR2 = True

CACHE_REDIS = rediscache://127.0.0.1:6379:1?client_class=redis_cache.client.DefaultClient&password=secret
CACHE_URL = memcache://127.0.0.1:11211

DATABASE_MYSQL_GIS_URL = mysqlgis://user:password@127.0.0.1/some_database
DATABASE_MYSQL_URL = mysql://bea6eb025ca0d8:69772142@us-cdbr-east.cleardb.com/heroku_97681db3eff7580?reconnect=true
DATABASE_SQLITE_URL = sqlite:////full/path/to/your/database/file.sqlite
DATABASE_URL = postgres://uf07k1i6d8ia0v:wegauwhgeuioweg@ec2-107-21-253-135.compute-1.amazonaws.com:5431/d8r82722r2kuvn

DICT_VAR = foo=bar,test=on

'''
        self.assertEqual(stream.getvalue(), expected)

def load_suite():

    test_suite = unittest.TestSuite()
    cases = [
        EnvTests, FileEnvTests, OsEnvironTests, SchemaEnvTests,
        DatabaseTestSuite, CacheTestSuite, EmailTests, InterpolationTests,
        PrettyPrintTests, DictionaryInterfaceTests, MoreInterpolationTests
    ]
    for case in cases:
        test_suite.addTest(unittest.makeSuite(case))
    return test_suite

if __name__ == "__main__":

    try:
        if sys.argv[1] == '-o':
            for key, value in BaseTests.generateData().items():
                print("{0}={1}".format(key, value))
            sys.exit()
    except IndexError:
        pass

    unittest.TextTestRunner().run(load_suite())




"""
12factor inspired environment configuration
"""
from __future__ import unicode_literals
import os
import sys
import re
import json
import warnings
import glob
import collections
import logging

logger = logging.getLogger(__file__)

try:
    import urllib.parse as urlparse
except ImportError:
    # Python <= 2.6
    import urlparse


if sys.version < '3':
    text_type = unicode
else:
    text_type = str
    basestring = str

from .interpolation import (
    resolve, resolve_files, StringTemplate, is_variable, interpolated
)

__author__ = 'joke2k'


# return int if possible
_cast_int = lambda v: int(v) if isinstance(v, basestring) and v.isdigit() else v
# return str if possibile
_cast_str = lambda v: str(v) if isinstance(v, basestring) else v


class NoValue(object):
    def __repr__(self):
        return '<{0}>'.format(self.__class__.__name__)


class Environment(collections.MutableMapping):
    """Provide schema-based lookups of environment variables so that each
    caller doesn't have to pass in `cast` and `default` parameters.

    Usage:::

        env = Environment(MAIL_ENABLED=bool, SMTP_LOGIN=(str, 'DEFAULT'))
        if env('MAIL_ENABLED'):
            ...
    """

    NOTSET = NoValue()
    BOOLEAN_TRUE_STRINGS = ('true', 'on', 'ok', 'y', 'yes', '1')
    URL_CLASS = urlparse.ParseResult
    DEFAULT_DATABASE_ENV = 'DATABASE_URL'
    DB_SCHEMES = {
        'postgres': 'django.db.backends.postgresql_psycopg2',
        'postgresql': 'django.db.backends.postgresql_psycopg2',
        'psql': 'django.db.backends.postgresql_psycopg2',
        'pgsql': 'django.db.backends.postgresql_psycopg2',
        'postgis': 'django.contrib.gis.db.backends.postgis',
        'mysql': 'django.db.backends.mysql',
        'mysql2': 'django.db.backends.mysql',
        'mysqlgis': 'django.contrib.gis.db.backends.mysql',
        'spatialite': 'django.contrib.gis.db.backends.spatialite',
        'sqlite': 'django.db.backends.sqlite3',
        'ldap': 'ldapdb.backends.ldap',
    }
    _DB_BASE_OPTIONS = ['CONN_MAX_AGE', 'ATOMIC_REQUESTS', 'AUTOCOMMIT']

    DEFAULT_CACHE_ENV = 'CACHE_URL'
    CACHE_SCHEMES = {
        'dbcache': 'django.core.cache.backends.db.DatabaseCache',
        'dummycache': 'django.core.cache.backends.dummy.DummyCache',
        'filecache': 'django.core.cache.backends.filebased.FileBasedCache',
        'locmemcache': 'django.core.cache.backends.locmem.LocMemCache',
        'memcache': 'django.core.cache.backends.memcached.MemcachedCache',
        'pymemcache': 'django.core.cache.backends.memcached.PyLibMCCache',
        'rediscache': 'redis_cache.cache.RedisCache',
        'redis': 'redis_cache.cache.RedisCache',
    }
    _CACHE_BASE_OPTIONS = ['TIMEOUT', 'KEY_PREFIX', 'VERSION', 'KEY_FUNCTION']

    DEFAULT_EMAIL_ENV = 'EMAIL_URL'
    EMAIL_SCHEMES = {
        'smtp': 'django.core.mail.backends.smtp.EmailBackend',
        'smtps': 'django.core.mail.backends.smtp.EmailBackend',
        'consolemail': 'django.core.mail.backends.console.EmailBackend',
        'filemail': 'django.core.mail.backends.filebased.EmailBackend',
        'memorymail': 'django.core.mail.backends.locmem.EmailBackend',
        'dummymail': 'django.core.mail.backends.dummy.EmailBackend'
    }
    _EMAIL_BASE_OPTIONS = ['EMAIL_USE_TLS', ]

    DEFAULT_SEARCH_ENV = 'SEARCH_URL'
    SEARCH_SCHEMES = {
        "elasticsearch": "haystack.backends.elasticsearch_backend.ElasticsearchSearchEngine",
        "solr": "haystack.backends.solr_backend.SolrEngine",
        "whoosh": "haystack.backends.whoosh_backend.WhooshEngine",
        "simple": "haystack.backends.simple_backend.SimpleEngine",
    }
    RESERVED_PATTERN = re.compile('key|secret|passwd|password')



    def __init__(self, init=None, **schema):
        if init is None:
            init = os.environ
        self._environ = init
        self._schema = schema
        self._resolved = None

    def __call__(self, var, cast=None, default=NOTSET):
        return self.get_value(var, cast=cast, default=default)

    # Shortcuts

    def str(self, var, default=NOTSET):
        """
        :rtype: str
        """
        return self.get_value(var, default=default)

    def unicode(self, var, default=NOTSET):
        """Helper for python2
        :rtype: unicode
        """
        return self.get_value(var, cast=text_type, default=default)

    ###########################################################################
    #    Dictionary Interface                                                 #
    ###########################################################################
    def __getitem__(self, key):
        return self.get_value(key)

    def __setitem__(self, key, value):
        if self._resolved is not None and is_variable(val):
            del self._resolved
            self._resolved = None
        self._environ[key] = value

    def __delitem__(self, key):
        del self._environ[key]

    def __iter__(self):
        return iter(self._environ)

    def __len__(self):
        return len(self._environ)

    def get(self, key, default=NOTSET):
        return self.get_value(key, default=default)

    def copy(self):
        return self.__class__(self._environ.copy(), **self._schema)
    ###########################################################################

    def bool(self, var, default=NOTSET):
        """
        :rtype: bool
        """
        return self.get_value(var, cast=bool, default=default)

    def int(self, var, default=NOTSET):
        """
        :rtype: int
        """
        return self.get_value(var, cast=int, default=default)

    def float(self, var, default=NOTSET):
        """
        :rtype: float
        """
        return self.get_value(var, cast=float, default=default)

    def json(self, var, default=NOTSET):
        """
        :returns: Json parsed
        """
        return self.get_value(var, cast=json.loads, default=default)

    def list(self, var, cast=None, default=NOTSET):
        """
        :rtype: list
        """
        return self.get_value(var, cast=list if not cast else [cast], default=default)

    def dict(self, var, cast=dict, default=NOTSET):
        """
        :rtype: dict
        """
        return self.get_value(var, cast=cast, default=default)

    def url(self, var, default=NOTSET):
        """
        :rtype: urlparse.ParseResult
        """
        return self.get_value(var, cast=urlparse.urlparse, default=default)

    def db_url(self, var=DEFAULT_DATABASE_ENV, default=NOTSET, engine=None):
        """Returns a config dictionary, defaulting to DATABASE_URL.

        :rtype: dict
        """
        return self.db_url_config(self.get_value(var, default=default), engine=engine)
    db=db_url

    def cache_url(self, var=DEFAULT_CACHE_ENV, default=NOTSET, backend=None):
        """Returns a config dictionary, defaulting to CACHE_URL.

        :rtype: dict
        """
        return self.cache_url_config(self.url(var, default=default), backend=backend)
    cache=cache_url

    def email_url(self, var=DEFAULT_EMAIL_ENV, default=NOTSET, backend=None):
        """Returns a config dictionary, defaulting to EMAIL_URL.

        :rtype: dict
        """
        return self.email_url_config(self.url(var, default=default), backend=backend)

    def search_url(self, var=DEFAULT_SEARCH_ENV, default=NOTSET, engine=None):
        """Returns a config dictionary, defaulting to SEARCH_URL.

        :rtype: dict
        """
        return self.search_url_config(self.url(var, default=default), engine=engine)

    def resolved(self):
        if self._resolved is None:
            self._resolved = self.__class__(
                interpolated(self._environ), **self._schema
            )
        return self._resolved

    def get_value(self, var, cast=None, default=NOTSET):
        """Return value for given environment variable.

        :param var: Name of variable.
        :param cast: Type to cast return value as.
        :param default: If var not present in environ, return this instead.

        :returns: Value from environment or default (if set)
        """
        logger.debug("get '{0}' casted as '{1}' with default '{2}'".format(var, cast, default))
        try:
            var_info = self._schema[var]
        except KeyError:
            pass
        else:
            try:
                has_default = len(var_info) == 2
            except TypeError:
                has_default = False
            if has_default:
                if not cast:
                    cast = var_info[0]

                if default is self.NOTSET:
                    try:
                        default = var_info[1]
                    except IndexError:
                        pass
            else:
                if not cast:
                    cast = var_info
        try:
            value = self._environ[var]
        except KeyError:
            if default is self.NOTSET:
                #error_msg = "Set the {0} environment variable".format(var)
                raise
            value = default
        if value is not default:
            value = self.parse_value(value, cast)
        if is_variable(value):
            try:
                return self.resolved()._environ[var]
            except KeyError:
                pass
        return value

    # Class and static methods

    def parse_value(self, value, cast):
        """Parse and cast provided value

        :param value: Stringed value.
        :param cast: Type to cast return value as.

        :returns: Casted value
        """
        if value is None:
            return value
        elif cast is None:
            return value
        elif cast is bool:
            try:
                value = int(value) != 0
            except ValueError:
                value = value.lower() in self.BOOLEAN_TRUE_STRINGS
        elif isinstance(cast, list):
            value = list(map(cast[0], [x for x in value.split(',') if x]))
        elif isinstance(cast, dict):
            key_cast = cast.get('key', str)
            value_cast = cast.get('value', text_type)
            value_cast_by_key = cast.get('cast', dict())
            value = dict(map(
                lambda kv: (key_cast(kv[0]), self.parse_value(kv[1], value_cast_by_key.get(kv[0], value_cast))),
                [val.split('=') for val in value.split(';') if val]
            ))
        elif cast is dict:
            value = dict([val.split('=') for val in value.split(',') if val])
        elif cast is list:
            value = [x for x in value.split(',') if x]
        elif cast is float:
            # clean string
            float_str = re.sub(r'[^\d,\.]', '', value)
            # split for avoid thousand separator and different locale comma/dot symbol
            parts = re.split(r'[,\.]', float_str)
            if len(parts) == 1:
                float_str = parts[0]
            else:
                float_str = "{0}.{1}".format(''.join(parts[0:-1]), parts[-1])
            value = float(float_str)
        else:
            value = cast(value)
        return value

    def db_url_config(self, url, engine=None):
        """Pulled from DJ-Database-URL, parse an arbitrary Database URL.
        Support currently exists for PostgreSQL, PostGIS, MySQL and SQLite.

        SQLite connects to file based databases. The same URL format is used, omitting the hostname,
        and using the "file" portion as the filename of the database.
        This has the effect of four slashes being present for an absolute file path:

        >>> from environ import Environment
        >>> Environment.db_url_config('sqlite:////full/path/to/your/file.sqlite')
        {'ENGINE': 'django.db.backends.sqlite3', 'HOST': None, 'NAME': '/full/path/to/your/file.sqlite', 'PASSWORD': None, 'PORT': None, 'USER': None}
        >>> Environment.db_url_config('postgres://uf07k1i6d8ia0v:wegauwhgeuioweg@ec2-107-21-253-135.compute-1.amazonaws.com:5431/d8r82722r2kuvn')
        {'ENGINE': 'django.db.backends.postgresql_psycopg2', 'HOST': 'ec2-107-21-253-135.compute-1.amazonaws.com', 'NAME': 'd8r82722r2kuvn', 'PASSWORD': 'wegauwhgeuioweg', 'PORT': 5431, 'USER': 'uf07k1i6d8ia0v'}

        """
        if not isinstance(url, self.URL_CLASS):
            if url == 'sqlite://:memory:':
                # this is a special case, because if we pass this URL into
                # urlparse, urlparse will choke trying to interpret "memory"
                # as a port number
                return {
                    'ENGINE': self.DB_SCHEMES['sqlite'],
                    'NAME': ':memory:'
                }
                # note: no other settings are required for sqlite
            url = urlparse.urlparse(url)

        config = {}

        # Remove query strings.
        path = url.path[1:]
        path = path.split('?', 2)[0]

        # if we are using sqlite and we have no path, then assume we
        # want an in-memory database (this is the behaviour of sqlalchemy)
        if url.scheme == 'sqlite' and path == '':
            path = ':memory:'
        if url.scheme == 'ldap':
            path = '{scheme}://{hostname}'.format(scheme=_cast_str(url.scheme), hostname=_cast_str(url.hostname))
            if url.port:
                path += ':{port}'.format(port=_cast_str(url.port))

        # Update with environment configuration.
        config.update({
            'NAME': path,
            'USER': _cast_str(url.username),
            'PASSWORD': _cast_str(url.password),
            'HOST': _cast_str(url.hostname),
            'PORT': _cast_int(url.port),
        })

        if url.query:
            config_options = {}
            for k, v in urlparse.parse_qs(url.query).items():
                if k.upper() in self._DB_BASE_OPTIONS:
                    config.update({k.upper(): _cast_int(v[0])})
                else:
                    config_options.update({k: _cast_int(v[0])})
            config['OPTIONS'] = config_options

        if engine:
            config['ENGINE'] = engine
        if url.scheme in Environment.DB_SCHEMES:
            config['ENGINE'] = Environment.DB_SCHEMES[url.scheme]

        if not config.get('ENGINE', False):
            warnings.warn("Engine not recognized from url: {0}".format(config))
            return {}

        return config

    def cache_url_config(self, url, backend=None):
        """Pulled from DJ-Cache-URL, parse an arbitrary Cache URL.

        :param url:
        :param overrides:
        :return:
        """
        url = urlparse.urlparse(url) if not isinstance(url, self.URL_CLASS) else url

        location = url.netloc.split(',')
        if len(location) == 1:
            location = location[0]

        config = {
            'BACKEND': self.CACHE_SCHEMES[url.scheme],
            'LOCATION': location,
        }

        if url.scheme == 'filecache':
            config.update({
                'LOCATION': _cast_str(url.netloc + url.path),
            })

        if url.path and url.scheme in ['memcache', 'pymemcache', 'rediscache']:
            config.update({
                'LOCATION': 'unix:' + url.path,
            })

        if url.query:
            config_options = {}
            for k, v in urlparse.parse_qs(url.query).items():
                opt = {k.upper(): _cast_int(v[0])}
                if k.upper() in self._CACHE_BASE_OPTIONS:
                    config.update(opt)
                else:
                    config_options.update(opt)
            config['OPTIONS'] = config_options

        if backend:
            config['BACKEND'] = backend

        return config

    def email_url_config(self, url, backend=None):
        """Parses an email URL."""

        config = {}

        url = urlparse.urlparse(url) if not isinstance(url, self.URL_CLASS) else url

        # Remove query strings
        path = url.path[1:]
        path = path.split('?', 2)[0]

        # Update with environment configuration
        config.update({
            'EMAIL_FILE_PATH': path,
            'EMAIL_HOST_USER': _cast_str(url.username),
            'EMAIL_HOST_PASSWORD': _cast_str(url.password),
            'EMAIL_HOST': _cast_str(url.hostname),
            'EMAIL_PORT': _cast_int(url.port),
        })

        if backend:
            config['EMAIL_BACKEND'] = backend
        elif url.scheme in self.EMAIL_SCHEMES:
            config['EMAIL_BACKEND'] = self.EMAIL_SCHEMES[url.scheme]

        if url.scheme == 'smtps':
            config['EMAIL_USE_TLS'] = True
        else:
            config['EMAIL_USE_TLS'] = False

        if url.query:
            config_options = {}
            for k, v in urlparse.parse_qs(url.query).items():
                opt = {k.upper(): _cast_int(v[0])}
                if k.upper() in self._EMAIL_BASE_OPTIONS:
                    config.update(opt)
                else:
                    config_options.update(opt)
            config['OPTIONS'] = config_options

        return config

    def search_url_config(self, url, engine=None):
        config = {}

        url = urlparse.urlparse(url) if not isinstance(url, self.URL_CLASS) else url

        # Remove query strings.
        path = url.path[1:]
        path = path.split('?', 2)[0]

        if url.scheme in self.SEARCH_SCHEMES:
            config["ENGINE"] = self.SEARCH_SCHEMES[url.scheme]

        if path.endswith("/"):
            path = path[:-1]

        split = path.rsplit("/", 1)

        if len(split) > 1:
            path = split[:-1]
            index = split[-1]
        else:
            path = ""
            index = split[0]

        config.update({
            "URL": urlparse.urlunparse(("http",) + url[1:2] + (path,) + url[3:]),
            "INDEX_NAME": index,
            })

        if path:
            config.update({
                "PATH": path,
            })

        if engine:
            config['ENGINE'] = engine

        return config

    def read_env(self, env_file=None, **overrides):
        """Read a .env file into os.environ.

        If not given a path to a dotenv path, does filthy magic stack backtracking
        to find manage.py and then find the dotenv.

        http://www.wellfireinteractive.com/blog/easier-12-factor-django/

        https://gist.github.com/bennylope/2999704
        """
        if env_file is None:
            frame = sys._getframe()
            env_file = os.path.join(os.path.dirname(frame.f_back.f_code.co_filename), '.env')
            if not os.path.exists(env_file):
                warnings.warn("not reading %s - it doesn't exist." % env_file)
                return

        def iterator(iterable):
            for line in iterable:
                m1 = re.match(r'\A([A-Za-z_0-9]+)=(.*)\Z', line)
                if m1:
                    key, val = m1.group(1), m1.group(2)
                    m2 = re.match(r"\A'(.*)'\Z", val)
                    if m2:
                        val = m2.group(1)
                    m3 = re.match(r'\A"(.*)"\Z', val)
                    if m3:
                        val = re.sub(r'\\(.)', r'\1', m3.group(1))
                    yield key, text_type(val)

        self.read(env_file, overrides=overrides, iterator=iterator)

    def read(self, files, defaults=None, overrides=None, iterator=None):
        """
        Populate the environment dictionary from one or more files.

        Interpolates values in the format $var or ${var}. If an interpolation
        target is missing, a KeyError is raised; and if any key is an invalid
        Python variable, a ValueError is raised.

        The default file iterator will determine key/value pairs by splitting
        lines on both '=' and ':='.
        """
        if isinstance(files, basestring) or hasattr(files, 'read'):
            files = [files]
        self._environ.update(resolve_files(files, defaults, overrides, iterator))

    def pprint(self, stream=sys.stdout, maxlines=-1, safe=True, encoding='utf-8'):

        def is_reserved(key):
            return bool(self.RESERVED_PATTERN.search(key.lower()))

        from itertools import groupby
        env = self._environ
        line = -1 * maxlines
        for _, group in groupby(sorted(env.keys()), lambda X: X.split('_')[0]):
            if line == 0:
                break
            stream.write(b'\n')
            for key in group:
                if line == 0:
                    break
                if safe and is_reserved(key):
                    val = '*'*8
                else:
                    val = env[key]
                stream.write(('%s = %s' % (key, val)).strip().encode(encoding))
                stream.write(b'\n')
                line += 1
        stream.write(b'\n')

environ = Environment()


def register_scheme(scheme):
    for method in filter(lambda s: s.startswith('uses_'), dir(urlparse)):
        getattr(urlparse, method).append(scheme)

# Register database and cache schemes in URLs.
_SCHEMES = []
for schtype in ['DB', 'CACHE', 'SEARCH', 'EMAIL']:
    _SCHEMES.extend(getattr(Environment, schtype + '_SCHEMES').keys())
for schema in _SCHEMES:
    register_scheme(schema)




from __future__ import unicode_literals
import os
from setuptools import setup, find_packages

import musette

def read(filepath):
    return open(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), filepath)
    ).read()

README = read('README.rst')

version = ".".join(map(str, musette.__version__))
author = musette.__author__
description = musette.__doc__

setup(name='musette',
      version=version,
      description=description,
      long_description=README,
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Intended Audience :: Information Technology',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 3',
          'Topic :: Software Development :: Libraries :: Python Modules',
          'Topic :: Utilities',
          'License :: OSI Approved :: MIT License',
          'Framework :: Django'
      ],
      keywords='django environment variables 12factor os.environ',
      author=author,
      author_email='gmflanagan@outlook.com',
      url='http://github.com/averagehuman/musette',
      license='MIT License',
      packages=find_packages(),
      include_package_data=True,
      test_suite='musette.test.load_suite',
      zip_safe=False,
)

